"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCompilerPluginByTarget = exports.getAllCompilerPlugins = exports.getAllCompilerTargets = exports.setCompilerPlugins = exports.getComposedCompilerPlugins = exports.PluginSjsFileTypes = exports.PluginScriptFileTypes = exports.PluginConfigFileTypes = exports.PluginStyleFileTypes = exports.PluginTemplateFileTypes = exports.Targets = void 0;
const tslib_1 = require("tslib");
/**
 * 载入所有编译插件
 * 供后续使用
 */
const alipayCompiler = tslib_1.__importStar(require("@morjs/plugin-compiler-alipay"));
const baiduCompiler = tslib_1.__importStar(require("@morjs/plugin-compiler-baidu"));
const bytedanceCompiler = tslib_1.__importStar(require("@morjs/plugin-compiler-bytedance"));
const dingdingCompiler = tslib_1.__importStar(require("@morjs/plugin-compiler-dingding"));
const kuaishouCompiler = tslib_1.__importStar(require("@morjs/plugin-compiler-kuaishou"));
const qqCompiler = tslib_1.__importStar(require("@morjs/plugin-compiler-qq"));
const taobaoCompiler = tslib_1.__importStar(require("@morjs/plugin-compiler-taobao"));
const webCompiler = tslib_1.__importStar(require("@morjs/plugin-compiler-web"));
const wechatCompiler = tslib_1.__importStar(require("@morjs/plugin-compiler-wechat"));
/**
 * 默认支持的 插件列表
 */
const DEFAULT_ALL_PLUGINS = [
    alipayCompiler,
    wechatCompiler,
    baiduCompiler,
    bytedanceCompiler,
    qqCompiler,
    taobaoCompiler,
    dingdingCompiler,
    kuaishouCompiler,
    webCompiler
];
exports.Targets = [
    alipayCompiler.target,
    wechatCompiler.target,
    baiduCompiler.target,
    bytedanceCompiler.target,
    qqCompiler.target,
    taobaoCompiler.target,
    dingdingCompiler.target,
    webCompiler.target
];
// 插件中支持的所有文件类型
exports.PluginTemplateFileTypes = Array.from(new Set(DEFAULT_ALL_PLUGINS.map((p) => p.fileType.template)));
exports.PluginStyleFileTypes = Array.from(new Set(DEFAULT_ALL_PLUGINS.map((p) => p.fileType.style)));
exports.PluginConfigFileTypes = Array.from(new Set(DEFAULT_ALL_PLUGINS.map((p) => p.fileType.config)));
exports.PluginScriptFileTypes = Array.from(new Set(DEFAULT_ALL_PLUGINS.map((p) => p.fileType.script)));
exports.PluginSjsFileTypes = Array.from(new Set(DEFAULT_ALL_PLUGINS.map((p) => p.fileType.sjs)));
const PLUGIN_PROPS = [
    'target',
    'targetDescription',
    'fileType',
    'compileModuleKind',
    'compileScriptTarget',
    'compileSjsTarget',
    'globalObject',
    'resolveMainFields',
    'defaultNpmMiniProgramDist',
    'defaultConditionalFileExt',
    'defaultOutputDir',
    'projectConfigFiles',
    'supportGlobalComponents',
    'templateSingleTagNames',
    'templateSingleTagClosingType',
    'sjsTagName',
    'sjsSrcAttrName',
    'sjsModuleAttrName',
    'isSupportSjsContent',
    'templateDirectives',
    'templateProcessor',
    'getRuntimeFiles',
    'Plugin'
];
// 插件集合
let ComposedPlugins;
// 支持的 插件列表
let AllPlugins;
// 支持的 targets
let AllTargets;
/**
 * 获取 ComposedPlugins
 */
function getComposedCompilerPlugins() {
    return ComposedPlugins;
}
exports.getComposedCompilerPlugins = getComposedCompilerPlugins;
/**
 * 加载所有的插件, 并按照 ComposedCompilerPluginResult 的方式组装
 * 允许外部更新 ComposedPlugins
 */
function setCompilerPlugins(plugins) {
    const result = {};
    const targets = [];
    for (const plugin of plugins) {
        targets.push(plugin.target);
        for (const prop of PLUGIN_PROPS) {
            result[prop] = result[prop] || {};
            result[prop][plugin['target']] = plugin[prop];
        }
    }
    // 保存插件清单
    AllPlugins = [].concat(plugins);
    // 保存 targets
    AllTargets = targets;
    // 更新
    ComposedPlugins = result;
}
exports.setCompilerPlugins = setCompilerPlugins;
/**
 * 返回所有支持的 编译目标
 */
function getAllCompilerTargets() {
    return AllTargets;
}
exports.getAllCompilerTargets = getAllCompilerTargets;
/**
 * 获取所有支持的插件清单
 */
function getAllCompilerPlugins() {
    return AllPlugins;
}
exports.getAllCompilerPlugins = getAllCompilerPlugins;
/**
 * 根据 target 获取匹配的编译插件
 * @param target - 目标平台
 * @returns 匹配的编译插件
 */
function getCompilerPluginByTarget(target) {
    for (const plugin of AllPlugins) {
        if (plugin.target === target)
            return plugin;
    }
}
exports.getCompilerPluginByTarget = getCompilerPluginByTarget;
// 初始化
if (!ComposedPlugins)
    setCompilerPlugins(DEFAULT_ALL_PLUGINS);
//# sourceMappingURL=compilerPlugins.js.map