"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.preprocess = exports.COMPILE_LOADERS = exports.applyDefaults = void 0;
const tslib_1 = require("tslib");
const compile_1 = require("./compile");
const compilerPlugins_1 = require("./compilerPlugins");
const config_1 = require("./config");
Object.defineProperty(exports, "applyDefaults", { enumerable: true, get: function () { return config_1.applyDefaults; } });
Object.defineProperty(exports, "COMPILE_LOADERS", { enumerable: true, get: function () { return config_1.LOADERS; } });
const constants_1 = require("./constants");
const entries_1 = require("./entries");
const cssClassnameCompress_1 = require("./experiments/cssClassnameCompress");
const trimRuntimeHelpers_1 = require("./experiments/trimRuntimeHelpers");
const configParserPlugin_1 = require("./parsers/configParserPlugin");
const scriptParserPlugin_1 = require("./parsers/scriptParserPlugin");
const sjsParserPlugin_1 = require("./parsers/sjsParserPlugin");
const styleParserPlugin_1 = require("./parsers/styleParserPlugin");
const templateParserPlugin_1 = require("./parsers/templateParserPlugin");
const aliasSupportPlugin_1 = require("./plugins/aliasSupportPlugin");
const assetsManipulatePlugin_1 = require("./plugins/assetsManipulatePlugin");
const defineSupportPlugin_1 = require("./plugins/defineSupportPlugin");
const dynamicRequireSupportPlugin_1 = require("./plugins/dynamicRequireSupportPlugin");
const emitDeclarationsPlugin_1 = require("./plugins/emitDeclarationsPlugin");
const generateComposedAppJsonFilePlugin_1 = require("./plugins/generateComposedAppJsonFilePlugin");
const injectGetAppPlugin_1 = require("./plugins/injectGetAppPlugin");
const moduleSharingAndConsumingPlugin_1 = require("./plugins/moduleSharingAndConsumingPlugin");
const optimizeSplitChunksPlugin_1 = require("./plugins/optimizeSplitChunksPlugin");
const phantomDependencyPlugin_1 = require("./plugins/phantomDependencyPlugin");
const preRuntimeDetectionPlugin_1 = require("./plugins/preRuntimeDetectionPlugin");
const processComponentsPropsFunctionPlugin_1 = require("./plugins/processComponentsPropsFunctionPlugin");
const progressPlugin_1 = require("./plugins/progressPlugin");
const runtimeInjectPlugin_1 = require("./plugins/runtimeInjectPlugin");
const codePreprocessor_1 = require("./preprocessors/codePreprocessor");
Object.defineProperty(exports, "preprocess", { enumerable: true, get: function () { return codePreprocessor_1.preprocess; } });
tslib_1.__exportStar(require("./compilerPlugins"), exports);
tslib_1.__exportStar(require("./constants"), exports);
const APPLY_DEFAULTS_TO_USER_CONFIG_STAGE = -500;
const BUILD_WEBPACK_CONFIG_STAGE = -100;
/**
 * 编译功能
 * 单独抽出来一个类, 方便变量局部私有
 */
class MorCompile {
    constructor(name, runner) {
        this.name = name;
        this.runner = runner;
        this.addCompilerMethods();
        // 应用 experiments 插件
        new trimRuntimeHelpers_1.TrimRuntimeHelperPlugin().apply(runner);
        new cssClassnameCompress_1.CSSClassNameCompressPlugin().apply(runner);
        // 应用 declaration 生成支持插件
        new emitDeclarationsPlugin_1.EmitDeclarationsPlugin().apply(runner);
        // 模块共享与消费插件
        new moduleSharingAndConsumingPlugin_1.ModuleSharingAndConsumingPlugin().apply(runner);
        // app.json 编译和集成更新功能
        new generateComposedAppJsonFilePlugin_1.GenerateComposedAppJsonFilePlugin().apply(runner);
        // 应用 编译相关 插件
        new progressPlugin_1.ProgressPlugin().apply(runner);
        new assetsManipulatePlugin_1.AssetsManipulatePlugin().apply(runner);
        new injectGetAppPlugin_1.InjectGetAppPlugin().apply(runner);
        new optimizeSplitChunksPlugin_1.OptimizeSplitChunksPlugin().apply(runner);
        new runtimeInjectPlugin_1.RuntimeInjectPlugin().apply(runner);
        new dynamicRequireSupportPlugin_1.DynamicRequireSupportPlugin().apply(runner);
        new aliasSupportPlugin_1.AliasSupportPlugin().apply(runner);
        new defineSupportPlugin_1.DefineSupportPlugin().apply(runner);
        new phantomDependencyPlugin_1.PhantomDependencyPlugin().apply(runner);
        new preRuntimeDetectionPlugin_1.PreRuntimeDetectionPlugin().apply(runner);
        new processComponentsPropsFunctionPlugin_1.ProcessComponentsPropsFunctionPlugin().apply(runner);
        // 应用 parser 插件
        new configParserPlugin_1.ConfigParserPlugin().apply(runner);
        new scriptParserPlugin_1.ScriptParserPlugin().apply(runner);
        new styleParserPlugin_1.StyleParserPlugin().apply(runner);
        new templateParserPlugin_1.TemplateParserPlugin().apply(runner);
        new sjsParserPlugin_1.SjsParserPlugin().apply(runner);
        // 应用编译插件
        Object.values((0, compilerPlugins_1.getComposedCompilerPlugins)().Plugin).forEach((Plugin) => {
            if (Plugin)
                new Plugin().apply(runner);
        });
        // 获取 webpackWrapper
        runner.hooks.webpackWrapper.tap(this.name, (webpackWrapper) => {
            this.webpackWrapper = webpackWrapper;
        });
        this.registerCli();
        this.applyDefaultUserConfig();
        runner.hooks.matchedCommand.tap(this.name, (command = {}) => {
            // 非 compile 命令不执行后续
            if (command.name !== constants_1.COMPILE_COMMAND_NAME)
                return;
            this.registerUserConfig();
            this.modifyUserConfig();
            this.buildWebpackConfig();
        });
    }
    /**
     * 添加编译共享出来的方法
     */
    addCompilerMethods() {
        const methods = this.runner.methods;
        /**
         * 支持获取编译器的默认 loaders
         */
        methods.register('getCompilerLoaders', function () {
            return config_1.LOADERS;
        });
        /**
         * 支持获取 composedPlugins
         */
        methods.register('getComposedCompilerPlugins', function () {
            return (0, compilerPlugins_1.getComposedCompilerPlugins)();
        });
    }
    // 注册 cli 支持, 基于 CompilerCliConfig 中的设定自动生成
    registerCli() {
        this.runner.hooks.cli.tap({
            name: this.name,
            stage: -100
        }, (cli) => {
            const compileCommand = cli.command(constants_1.COMPILE_COMMAND_NAME, '编译小程序工程');
            const targets = (0, compilerPlugins_1.getAllCompilerTargets)();
            const composedPlugins = (0, compilerPlugins_1.getComposedCompilerPlugins)();
            const longestTargetLength = targets.reduce((length, t) => (t.length >= length ? t.length : length), 0);
            // 编译命令的使用介绍
            const usage = `compile 编译命令\n\n  支持的小程序或应用类型 (target):\n    ${Object.keys(composedPlugins.targetDescription)
                .sort()
                .map((t) => `${t.padEnd(longestTargetLength)}  ${composedPlugins.targetDescription[t]}`)
                .join('\n    ')}`;
            compileCommand.usage(usage);
            for (const option in constants_1.CompilerCliConfig) {
                const optionSchema = constants_1.CompilerCliConfig[option];
                // 如未配置 cliOption 则跳过
                if (!optionSchema.cliOption)
                    continue;
                const options = {};
                // 命令行不设置默认值
                if ('default' in optionSchema)
                    options.default = optionSchema.default;
                // 拼接完整描述
                let optionDesc = optionSchema.name;
                if (optionSchema.desc)
                    optionDesc = `${optionDesc}, ${optionSchema.desc}`;
                if (optionSchema.valuesDesc) {
                    const valueDesc = typeof optionSchema.valuesDesc === 'function'
                        ? optionSchema.valuesDesc()
                        : optionSchema.valuesDesc;
                    optionDesc = `${optionDesc}, ${valueDesc}`;
                }
                compileCommand.option(optionSchema.cliOption, optionDesc, options);
            }
            // 执行 compile
            compileCommand.action(async () => await this.runCompile());
        });
    }
    // 基于命令行参数修改 userConfig
    modifyUserConfig() {
        const { config } = this.runner;
        this.runner.hooks.modifyUserConfig.tap(this.name, function (userConfig = {}, command) {
            return (0, config_1.modifyUserConfig)(config, userConfig, (command === null || command === void 0 ? void 0 : command.options) || {});
        });
    }
    // 应用用户默认配置
    applyDefaultUserConfig() {
        const { hooks, config } = this.runner;
        hooks.userConfigValidated.tapPromise({
            name: `${this.name}:applyDefaultUserConfig`,
            stage: APPLY_DEFAULTS_TO_USER_CONFIG_STAGE
        }, async () => {
            const userConfig = this.runner.userConfig;
            (0, config_1.applyDefaults)(config, userConfig);
            // 在 APPLY_DEFAULTS_TO_USER_CONFIG_STAGE 阶段初始化 entryBuidler
            // 并触发 entryBuilder hook
            await this.getEntryBuilder();
        });
    }
    // compile 支持的配置
    registerUserConfig() {
        const { hooks } = this.runner;
        hooks.registerUserConfig.tap(this.name, function (s, z) {
            const allTargets = (0, compilerPlugins_1.getAllCompilerTargets)();
            return s.merge(constants_1.CompilerUserConfigSchema).extend({
                /**
                 * 这里覆盖 target 的校验 用于允许通过外部自动追加编译插件
                 */
                target: z
                    .string()
                    .refine((val) => allTargets.includes(val), (val) => ({
                    message: `无效的值. 期望是 ${allTargets
                        .map((val) => `'${val}'`)
                        .join(' 或 ')}, 实际是 '${val}'`
                }))
                    .default(allTargets[0])
            });
        });
    }
    async getEntryBuilder() {
        if (this.entryBuilder)
            return this.entryBuilder;
        const entryBuilder = new entries_1.EntryBuilder(this.runner.config, this.runner.userConfig, this.webpackWrapper, this.runner.context, this.runner);
        /* 触发 entryBuilder hook */
        await this.runner.hooks.entryBuilder.promise(entryBuilder);
        this.entryBuilder = entryBuilder;
        return this.entryBuilder;
    }
    buildWebpackConfig() {
        const { hooks, config } = this.runner;
        hooks.userConfigValidated.tapPromise({
            name: `${this.name}:buildWebpackConfig`,
            // 越小越靠前
            stage: BUILD_WEBPACK_CONFIG_STAGE
        }, async () => {
            const userConfig = this.runner.userConfig;
            const entryBuilder = await this.getEntryBuilder();
            await (0, config_1.buildWebpackConfig)(config, userConfig, this.webpackWrapper, entryBuilder, this.runner);
        });
    }
    // 执行编译
    async runCompile() {
        const entryBuilder = await this.getEntryBuilder();
        await (0, compile_1.compile)(this.webpackWrapper, entryBuilder, this.runner);
    }
}
/**
 * 编译插件
 */
class MorCompilerPlugin {
    constructor() {
        this.name = 'MorCompilerPlugin';
    }
    apply(runner) {
        new MorCompile(this.name, runner);
    }
}
exports.default = MorCompilerPlugin;
//# sourceMappingURL=index.js.map