"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@morjs/utils");
const path_1 = require("path");
const constants_1 = require("../constants");
// 后置处理 loader
async function PostprocessLoader(fileContent, sourceMap) {
    const cb = this.async();
    const filePath = this.resourcePath;
    const { userConfig, runner, entryBuilder } = this.getOptions() || {};
    const { compileMode } = userConfig;
    const entry = entryBuilder.getEntryByFilePath(filePath);
    try {
        let result;
        // 在 preprocess 之前调用 插件
        if (runner.hooks.postprocessorParser.isUsed()) {
            result = await runner.hooks.postprocessorParser.promise(fileContent, {
                userConfig,
                loaderContext: this,
                fileInfo: {
                    path: filePath,
                    content: fileContent,
                    extname: entry ? entry.extname : (0, path_1.extname)(filePath),
                    entryName: entry ? entry.fullEntryName : undefined,
                    entryFileType: entry ? entry.entryFileType : undefined,
                    entryType: entry ? entry.entryType : utils_1.EntryType.unknown
                }
            });
        }
        else {
            result = fileContent;
        }
        // bundle 模式下所有文件均交给 webpack 处理并打包
        // transform 模式下, 手动处理 js entry, 绕过 webpack
        // 后续会在 webpack.compiler.hooks.processAssets 阶段重写 assets
        // 参见 compile.ts 文件
        if (compileMode !== constants_1.CompileModes.bundle &&
            entry &&
            entry.entryFileType === utils_1.EntryFileType.script &&
            entry.fullEntryName) {
            entryBuilder.setEntrySource(entry.fullEntryName, result, 'replace');
            cb(null, '');
        }
        else {
            cb(null, result, sourceMap);
        }
    }
    catch (error) {
        cb(error);
    }
}
exports.default = PostprocessLoader;
//# sourceMappingURL=postprocessLoader.js.map